#include <stdlib.h>
#include <stream.h>
#include <math.h>

#define MAXSTR 1024
#define PARALLEL 0
#define SINGLEPROC 1

int main(int argc, char **argv)
{
  float header[512];
  int m,n,k;
  int l,j,i;
  int ncount,cnt;
  int ns,ni,nj;
  int checkflg=0, version;
  float *data=NULL;
  int *sel=NULL;
  FILE *fm;
  FILE *fp;
  FILE *fs;
  char fpname[MAXSTR];
  char fmname[MAXSTR];
  char fsname[MAXSTR];
  char fake;
  

  if (argc<4 || argc > 5) {
    cout<<"Usage: bruk2mdd <nmrPipe stream> <MDD output> <MDD version> [<schedule>]\n";
    cout<<"                <nmrPipe stream> - file (not planes!) containing 3D nmrPipe format data\n";
    cout<<"                <MDD output>     - output file in MDD format\n";
    cout<<"                <MDD version>    - currently is 0 for parallel and 1 for single processor\n";
    cout<<"                <schedule>       - schedule (selection) file, optional but may be useful for a double check\n";
    exit(1);
  }
  
  strcpy(fpname,argv[1]);
  strcpy(fmname,argv[2]);
  if (1!=sscanf(argv[3],"%d",&version)) {
	  cerr<<"Could not determine output file version ["<<argv[3]<<"]. Exiting...\n";
    exit(1);
  }
  else {
    if ((version!=PARALLEL) && (version!=SINGLEPROC)) {
	    cerr<<"Could not determine output file version ("<<version<<"). Exiting...\n";
      exit(1);
    }
  }

  if (argc==5) {
    checkflg=1;
    strcpy(fsname,argv[4]);
  }
  
  
  if (NULL==(fp=fopen(fpname,"rb"))) {
    cerr<<"Could not open file "<<fpname<<" for reading. Exiting...\n";
    exit(2);
  }
  
  if (512!=fread(header,sizeof(float),512,fp)) {
    cerr<<"Could not read 512 floats into header from file "<<fpname<<". Exiting...\n";
    exit(3);
  }  
  
  k=header[15];    // dim 3 size
  n=header[428]*2; // dim 2 size
  m=header[99];    // dim 1 size
  
  cout<<"Sizes are X: "<<m<<"; Y: "<<n<<"; Z: "<<k<<"\n";
  
  if (NULL==(data=new float[m*n*k])) {
    cerr<<"Could not allocate "<<m*n*k*4<<" bytes of memory to store data. Exiting...\n";
    exit(4);
  }
    
  if (m*n*k!=fread(data,sizeof(float),m*n*k,fp)) {    
    cerr<<"Could not read "<<m*n*k<<" floats of data from file "<<fpname<<". Exiting...\n";
    exit(3);
  }
  if (0<fread(&fake,1,1,fp)) {
    cerr<<"There is still data in file "<<fpname<<" although "<<512+m*n*k<<" floats of header and data were read. Exiting..\n";
    exit(3);
  }
  fclose(fp);
  
  cout<<"checkflg="<<checkflg<<"\n";
  
  if (NULL==(sel=new int[n*k])) {
    cerr<<"Could not allocate "<<n*k*sizeof(int)<<" bytes of memory to store selection matrix. Exiting...\n";
    exit(4);
  }

  
  if (checkflg) {
    if (NULL==(fs=fopen(fsname,"r"))) {
      cerr<<"Could not open file "<<fsname<<" for reading. Exiting...\n";
      exit(2);
    }
    for(i=0; i<k; i++)
      for (j=0; j<n; j++)
        sel[i*n+j]=0;
 
    for(ns=fscanf(fs,"%d%d",&ni,&nj); ns==2; ns=fscanf(fs,"%d%d",&ni,&nj)) {
      sel[ 2*ni   *n +2*nj  ]=1;
      sel[(2*ni+1)*n +2*nj  ]=1;
      sel[ 2*ni   *n +2*nj+1]=1;
      sel[(2*ni+1)*n +2*nj+1]=1;
    }
  
    ncount=0;
    cnt=0;
  
    for (i=0; i<k; i++) 
      for (j=0; j<n; j++) {
        if (fabs(data[i*m*n+j*m])>1e-6) {
	  ncount++;
	  if (sel[i*n+j]==0)
	    cerr<<i<<" "<<j<<" recorded unexpectedly\n";
        }
        else
          if (sel[i*n+j]==1)
 	    cerr<<i<<" "<<j<<" expected but not recorded\n";
        if (sel[i*n+j])
          cnt++;
        }
  }
  else {
    ncount=0;
    cnt=0;
    for (i=0; i<k; i++) 
      for (j=0; j<n; j++) 
        if (fabs(data[i*m*n+j*m])>1e-6) {
	  sel[i*n+j]=1;
	  ncount++; cnt++;
	}
  
  }
  
  if (NULL==(fm=fopen(fmname,"w"))) {
    cerr<<"Could not open file "<<fmname<<" for writing. Exiting...\n";
    exit(2);
  }
  switch (version) {
    case PARALLEL:
      fprintf(fm,"sparse asc\n");
      fprintf(fm,"%d %d %d %d\n",m*cnt,m,n,k);
      for (i=0; i<k; i++) 
        for (j=0; j<n; j++) 
          if (sel[i*n+j]) 
            for (l=0; l<m; l++) 
	      fprintf(fm,"%d %d %d\n",l,m+j,m+n+i);
      break;
    case SINGLEPROC:
      fprintf(fm,"mdd sparse asc\n");
      fprintf(fm,"produced by %s from file %s\n",argv[0],fpname);
      if (checkflg) fprintf(fm,"checked against file %s for consistency\n",fsname);
      fprintf(fm,"$\n3 1\n");
      fprintf(fm,"%d %d %d %d\n",m*cnt,m,n,k);
      for (i=0; i<k; i++) 
        for (j=0; j<n; j++) 
          if (sel[i*n+j]) 
            for (l=0; l<m; l++) 
	      fprintf(fm,"%d %d %d\n",l,j,i);
      break;
    default:
      cerr<<"Wrong version ("<<version<<". Exiting...\n";
      exit(10);
  }
  
  for (i=0; i<k; i++) 
    for (j=0; j<n; j++) 
      if (sel[i*n+j]) 
        for (l=0; l<m; l++) 
	  fprintf(fm,"%g\n",data[i*m*n+j*m+l]);

  fclose(fm); 
  cerr<<"\ndone...\n";
  return 0;
}
