#include <qmessagebox.h>
#include <qtextstream.h>
#include <qfile.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <qstring.h>

#define MAXSTR 1024
#define GAMMA_C13 0.251449530
#define GAMMA_N15 0.101329118


void frmmdd::select()
{
    QFile qtmp("acqu");
    
    if (qtmp.exists()) 
	BrukerButtonClicked();
    else 
	VarianButtonClicked();
	
    radioButton5_clicked();
 	

}

void frmmdd::help()
{
    QMessageBox::information(this,"HELP",
		"Please check the README.pdf file for detailed instructions");
}

// First step function - inserting zero filled FIDs into the ser file
// Also, preparing the bruk2pipe conversion script, which can be edited in the second step

QString frmmdd::getpar(char *key, FILE *fp)
{
  char res[MAXSTR];
  char word[MAXSTR];
  char iline[MAXSTR];
  QString result;
  char *p;
  int found=0,n;


  rewind(fp);
  strcpy(res,"");
  for(p=fgets(iline, MAXSTR, fp);(p!=NULL && found==0);p=fgets(iline, MAXSTR, fp)) {
    sscanf(iline,"%s",word);
    if (0==strcmp(word,key)) {
      p=fgets(iline,MAXSTR,fp);
      found=1;
      sscanf(iline,"%s",word);
      n=strlen(word);
      strcpy(res,iline+n+1);
    }
  }
  result=res;
  return(result);
}


void frmmdd::InsertZeros()
{
    if (radioButton3->isChecked()) {
	InsertZerosBruker();
    }
    else if (radioButton4_2->isChecked()) {
	InsertZerosVarian();
    }
    else {
	QMessageBox::information( this, "Inserting zeros failed","Please choose the data source:\nBruker or Varian?" );
    }
}



void frmmdd::InsertZerosVarian()
{
  int sys=-1;
  QString sInFidName=leInputSer->text().ascii();
  QString sProcparName=leVDlist->text().ascii();
  QString sOutFidName=leOutSer->text().ascii();
  QString xName, yName, zName;
  QString sCommand;
  FILE *fp;
  QString sLine, sWord, sValue;
  QStringList sList;
  QFile qtmp;
  QTextStream qstream;
  
  int np=0, nimax=0, nimax2=0;
  double fSWx=0, fSWy=0, fSWz=0;
  double fOBSx=0, fOBSy=0, fOBSz=0;
  double fCARx=0, fCARy=0, fCARz=0;
  double dfrq=0, dfrq2=0;
  QString iQline;
  QString seqfil;
  QString sSWAP;
  int EA=0, aqORD=0;
  
  double fBF_h1=0, fBF_c13=0, fBF_n15=0;
  
  if (NULL==(fp=fopen("procpar","r"))) {
      QMessageBox::information( this, "Inserting zeros for Varian failed","Cannot open file:\nprocpar" );
      exit(2);
  }
    
  sCommand="sparsevarian_S " + sInFidName+" "+sProcparName+" "+sOutFidName;
  sys=system(sCommand);
  qDebug("sys=%d", sys);
  qDebug(sCommand);
  if(sys==0) QMessageBox::information( this, "Inserting zeros in FID","Program seems to have completed normally\nCheck the console window for messages" );
  else QMessageBox::information( this, "Inserting zeros in FID","Program does NOT seem to have completed normally\nCheck the console window for messages" );

  
  iQline=getpar("np",fp);
  qDebug(iQline);
  np=iQline.toInt();
  iQline=getpar("nimax",fp);
  nimax=iQline.toInt();
  iQline=getpar("nimax2",fp);
  nimax2=iQline.toInt();
  iQline=getpar("array",fp);
  iQline.remove('"');
  iQline.remove(' ');
  iQline.remove('\n');
  if (iQline=="phase2,phase") aqORD=0;
  else if (iQline=="phase,phase2") aqORD=1;
  else aqORD=-1;
  
  iQline=getpar("sfrq",fp);
  fOBSx=iQline.toDouble();
  iQline=getpar("reffrq",fp);
  fBF_h1=iQline.toDouble();
  fCARx=1e6*(fOBSx-fBF_h1)/fBF_h1;
  fBF_c13=GAMMA_C13*fBF_h1;    //IUPAC recommendation
  fBF_n15=GAMMA_N15*fBF_h1;    //IUPAC recommendation
  
  iQline=getpar("dfrq",fp);
  dfrq=iQline.toDouble();
  iQline=getpar("dfrq2",fp);
  dfrq2=iQline.toDouble();
  iQline=getpar("sw",fp);
  fSWx=iQline.toDouble();
  iQline=getpar("sw1",fp);
  fSWy=iQline.toDouble();
  iQline=getpar("sw2",fp);
  fSWz=iQline.toDouble();

  iQline=getpar("dg",fp);
  iQline.remove('"');
  sList=QStringList::split(';',iQline);
  sList=sList.grep("DIMENSION");
  QStringList::iterator it;
  for (it=sList.begin(); it!=sList.end(); ++it) {
      (*it).replace(':',' ');
      if (0<(*it).find("d2(d2)",0)) 
	  yName=(*it).section(' ',1,1);
      else if (0<(*it).find("d3(d3)",0))
	  zName=(*it).section(' ',1,1);
  }
  xName="H1X";
  if (yName=="1H") {
      yName="H1Y";
      fOBSy=fOBSx;
      fCARy=fCARx;
  }    
  else if (yName=="13C") {
      yName="C13Y";
      fOBSy=dfrq;
      fCARy=1e6*(dfrq-fBF_c13)/fBF_c13;
  }    
  else if (yName=="15N") {
      yName="N15Y";
      fOBSy=dfrq2;
      fCARy=1e6*(dfrq2-fBF_n15)/fBF_n15;
  }    
  
  if (zName=="1H") {
      zName="H1Z";
      fOBSz=fOBSx;
      fCARz=fCARx;
  }    
  else if (zName=="13C") {
      zName="C13Z";
      fOBSz=dfrq;
      fCARz=1e6*(dfrq-fBF_c13)/fBF_c13;
  }    
  else if (zName=="15N") {
      zName="N15Z";
      fOBSz=dfrq2;
      fCARz=1e6*(dfrq2-fBF_n15)/fBF_n15;
  }    
  
  iQline=getpar("seqfil",fp);
  iQline.remove('"');
  iQline.remove(' ');
  iQline.remove('\n');
  seqfil=iQline+".c";
  
  qtmp.setName(seqfil);
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      if (0<sLine.find("gradpulse(icosel*",0)) EA=1;
    }
    qtmp.close();
  }
   textEdit1->setText("#!/bin/csh");
  textEdit1->append("\n\n# In this step, you have to edit the Varian to nmrPipe conversion script");
  textEdit1->append("# Make sure that the following values are set properly: ");
  textEdit1->append("# Spectrometer Input should be ./"+sOutFidName);
  textEdit1->append("# Set the acquisition modes in each indirect dimension according to the experiment");
  textEdit1->append("# The most common are Echo-Antiecho/Complex for Z diemension and Complex for Y");
  textEdit1->append("# Any file permission errors in attempting to read the procpar file will");
  textEdit1->append("# result in 0 or -1 values for other parameters. Double check!");  
  textEdit1->append("# Carrier frequencies may be off. Double check!");
  sSWAP="-noaswap";
  iQline.sprintf("\n\nvar2pipe -in ./%s %s -aqORD %d     \\",sOutFidName.latin1(),sSWAP.latin1(),aqORD);
  textEdit1->append(iQline);
  iQline.sprintf(" -xN %10ld  -yN %10d  -zN %10d  \\", np,nimax*2,nimax2*2);
  textEdit1->append(iQline);
  iQline.sprintf(" -xT %10d  -yT %10d  -zT %10d  \\", np/2,nimax,nimax2);
  textEdit1->append(iQline);
  if ((EA>0) && (seqfil!="ghcch_tocsy_S.c")) textEdit1->append(" -xMODE Complex  -yMODE Complex  -zMODE Echo-AntiEcho \\");
  else      textEdit1->append(" -xMODE Complex  -yMODE Complex  -zMODE Complex \\");
  iQline.sprintf(" -xSW %9.3f  -ySW %9.3f  -zSW %9.3f  \\", fSWx,fSWy,fSWz);
  textEdit1->append(iQline);
  iQline.sprintf(" -xOBS %8.3f  -yOBS %8.3f  -zOBS %8.3f \\", fOBSx,fOBSy,fOBSz);
  textEdit1->append(iQline);
  iQline.sprintf(" -xCAR %8.3f  -yCAR %8.3f  -zCAR %8.3f \\",fCARx,fCARy,fCARz);
  textEdit1->append(iQline);
  iQline.sprintf(" -xLAB %8s  -yLAB %8s  -zLAB %8s  \\",xName.latin1(),yName.latin1(),zName.latin1());
  textEdit1->append(iQline);
  textEdit1->append(" -ndim        3  -aq2D          States                    \\");
  if (seqfil=="ghcch_tocsy_S.c") {
    textEdit1->append("| nmrPipe -fn MAC -macro $NMRTXT/var_ranceY.M -noRd -noWr \\");
    textEdit1->append("| nmrPipe -fn MAC -macro $NMRTXT/var_ranceZ.M -noRd -noWr \\");
    textEdit1->append("| pipe2xyz  -out ./data/test%03d.fid -verb -ov\n\n");
  }  
  else  textEdit1->append(" -out ./data/test%03d.fid -verb -ov\n\n");

    
 

}



void frmmdd::InsertZerosBruker()
{
  int sys=-1;
  QString sInSerName=leInputSer->text().ascii();
  QString sVDListName=leVDlist->text().ascii();
  QString sOutSerName=leOutSer->text().ascii();
  QString sNoPts;
  QString xName, yName, zName;
  QString sCommand;
  QFile qtmp;
  QString sLine, sWord, sValue;
  QTextStream qstream;
  
  int iTDx=0, iTDy=0, iTDz=0;
  int iDECIM=0, iDSPFVS=0;
  long iSERsize=0;
  int iNFIDs=0;
  long itrueTDx=0;
  double fSWx=0, fSWy=0, fSWz=0;
  double fOBSx=0, fOBSy=0, fOBSz=0;
  double fCARx=0, fCARy=0, fCARz=0;
  double fGRPDLY=0;
  QString iQline;
  QString sSWAP;
  int EA=0, SW_INF=0;
  
  double fBF_h1=0, fBF_c13=0, fBF_n15=0;
  double fINFfast=0, fINFslow=0;
  
  
  // Getting TD value (number of points) for the direct dimension
  
  qtmp.setName("acqu");
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      sWord=sLine.section(' ',0,0);
      if (sWord=="##$TD=") {
        sValue=sLine.section(' ',1,1);
        iTDx=sValue.toInt();
      }
    }
    qtmp.close();
  }

  
  // Getting the size of "ser" file
  qtmp.setName(sInSerName);
  iSERsize=qtmp.size();
  qDebug("iSERsize=%ld", iSERsize);
  
  // Getting the number of FIDs  
  qtmp.setName(sVDListName);
  iNFIDs=0;
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      iNFIDs++;
    }
    qtmp.close();
  }
  itrueTDx=iSERsize/(8*iNFIDs);
  qDebug("iNFIDs=%d", iNFIDs);
  qDebug("itrueTDx=%ld", itrueTDx);
  sNoPts.sprintf("%ld", itrueTDx);
 
  // inserting the zeros
  sCommand = "sparsebruker "+sInSerName+" "+sVDListName+" "+sOutSerName+" "+sNoPts;
  sys=system(sCommand);
  qDebug("sys=%d", sys);
  qDebug(sCommand);
  if(sys==0) QMessageBox::information( this, "Inserting zeros in SER","Program seems to have completed normally\nCheck the console window for messages" );
  else QMessageBox::information( this, "Inserting zeros in SER","Program does NOT seem to have completed normally\nCheck the console window for messages" );
  
  
  // finding other relevant parameters:
  // Is there Echo-Antiecho and if SW or INF is used for sweep width?
  qtmp.setName("pulseprogram");
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      sWord=sLine.section(' ',0,0);
      if (sWord[0]!=';')
        if (sLine.find("in0=inf1",0)>=0) SW_INF=1;
        else if (sLine.find("igrad EA",0)>=0) EA=1;
    }
    qtmp.close();
  }
  qDebug("EA= %d; SW_INF= %d",EA,SW_INF);
          
  //Parameters for acquisition dimension and other from acqus
  xName="H1X";
  qtmp.setName("acqus");
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      sWord=sLine.section(' ',0,0);
      sValue=sLine.section(' ',1,1);
      if (sWord=="##$BF1=") {
        fBF_h1=sValue.toDouble();
        fBF_c13=GAMMA_C13*fBF_h1;    //IUPAC recommendation
        fBF_n15=GAMMA_N15*fBF_h1;    //IUPAC recommendation
      }
      else if (sWord=="##$SW_h=") 
        fSWx=sValue.toDouble();
      else if (sWord=="##$SFO1=") {
        fOBSx=sValue.toDouble();
        fCARx=1e6*(fOBSx-fBF_h1)/fBF_h1;
      }
      else if (sWord=="##$DECIM=") 
        iDECIM=sValue.toInt();
      else if (sWord=="##$DSPFVS=") 
        iDSPFVS=sValue.toInt();
      else if (sWord=="##$GRPDLY=") 
        fGRPDLY=sValue.toDouble();
      else if ((sWord=="##$INF=") && (SW_INF==1)) {
        sLine=qstream.readLine();
        sValue=sLine.section(' ',1,1);
        fINFslow=sValue.toDouble();
        fSWz=1e6/fINFslow;
        sValue=sLine.section(' ',2,2);
        fINFfast=sValue.toDouble();
        fSWy=1e6/fINFfast;
       }
    }
    qtmp.close();
  }
  
  
  // TD for Y dimension
  qtmp.setName("acqu2");
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      sWord=sLine.section(' ',0,0);
      if (sWord=="##$TD=") {
        sValue=sLine.section(' ',1,1);
        iTDy=sValue.toInt();
      }
    }
    qtmp.close();
  }
  
  // SW, CAR and OBS for Y dimension
  qtmp.setName("acqu2s");
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      sWord=sLine.section(' ',0,0);
      sValue=sLine.section(' ',1,1);
      if (sWord=="##$SFO1=") {
        fOBSy=sValue.toDouble();
        if (fOBSx/fOBSy>9) {
          yName="N15Y";
          fCARy=1e6*(fOBSy-fBF_n15)/fBF_n15;
        }
        else if (fOBSx/fOBSy>3) {
          yName="C13Y";
          fCARy=1e6*(fOBSy-fBF_c13)/fBF_c13;
        }
        else if (fOBSx/fOBSy>0.9) {
          yName="H1Y";
          fCARy=fCARx;
        }
        else {
          yName="Y";
          fCARy=0;
        }
      }
      else if ((sWord=="##$SW_h=") && (SW_INF==0))
        fSWy=sValue.toDouble();
      
    }
    qtmp.close();
  }

  
  // TD for Z dimension
  qtmp.setName("acqu3");
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      sWord=sLine.section(' ',0,0);
      if (sWord=="##$TD=") {
       sValue=sLine.section(' ',1,1);
       iTDz=sValue.toInt();
      }
    }
    qtmp.close();
  }
  
  qtmp.setName("acqu3s");
  if (qtmp.open(IO_ReadOnly)) {
    qstream.setDevice(&qtmp);
    while (!(qstream.atEnd())) {
      sLine=qstream.readLine();
      sWord=sLine.section(' ',0,0);
      sValue=sLine.section(' ',1,1);
      if (sWord=="##$SFO1=") {
        fOBSz=sValue.toDouble();
        if (fOBSx/fOBSz>9) {
          zName="N15Z";
          fCARz=1e6*(fOBSz-fBF_n15)/fBF_n15;
        }
        else if (fOBSx/fOBSz>3) {
          zName="C13Z";
          fCARz=1e6*(fOBSz-fBF_c13)/fBF_c13;
        }
        else if (fOBSx/fOBSz>0.9) {
          zName="H1Z";
          fCARz=fCARx;
        }
        else {
          zName="Z";
          fCARz=0;
        }
      }
      else if ((sWord=="##$SW_h=") && (SW_INF==0))
        fSWz=sValue.toDouble();
      
    }
    qtmp.close();
  }

  
  
  
  
  textEdit1->setText("#!/bin/csh");
  textEdit1->append("\n\n# In this step, you have to edit the Bruker to nmrPipe conversion script");
  textEdit1->append("# Make sure that the following values are set properly: ");
  textEdit1->append("# Spectrometer Input should be ./"+sOutSerName);
  textEdit1->append("# Set the acquisition modes in each indirect dimension according to the experiment");
  textEdit1->append("# The most common are Echo-Antiecho for Y diemension and States-TPPI-N for Z");
  textEdit1->append("# Any file permission errors in attempting to read acqu* files will");
  textEdit1->append("# result in 0 or -1 values for other parameters. Double check!");
  if (iDSPFVS==20) sSWAP="-aswap";
  else sSWAP="-noaswap";
  iQline.sprintf("\n\nbruk2pipe -in ./%s -bad 0.0 %s -DMX -decim %d -dspfvs %d  -grpdly %lf\\",sOutSerName.latin1(),sSWAP.latin1(),iDECIM,iDSPFVS,fGRPDLY);
  textEdit1->append(iQline);
  iQline.sprintf(" -xN %10ld  -yN %10d  -zN %10d  \\", itrueTDx,iTDy,iTDz);
  textEdit1->append(iQline);
  iQline.sprintf(" -xT %10d  -yT %10d  -zT %10d  \\", iTDx/2,iTDy/2,iTDz/2);
  textEdit1->append(iQline);
  if (EA>0) textEdit1->append(" -xMODE DQD -yMODE Echo-AntiEcho -zMODE States-TPPI-N \\");
  else      textEdit1->append(" -xMODE DQD -yMODE   States-TPPI -zMODE States-TPPI-N \\");
  iQline.sprintf(" -xSW %9.3f  -ySW %9.3f  -zSW %9.3f  \\", fSWx,fSWy,fSWz);
  textEdit1->append(iQline);
  iQline.sprintf(" -xOBS %8.3f  -yOBS %8.3f  -zOBS %8.3f \\", fOBSx,fOBSy,fOBSz);
  textEdit1->append(iQline);
  iQline.sprintf(" -xCAR %8.3f  -yCAR %8.3f  -zCAR %8.3f \\",fCARx,fCARy,fCARz);
  textEdit1->append(iQline);
  iQline.sprintf(" -xLAB %8s  -yLAB %8s  -zLAB %8s  \\",xName.latin1(),yName.latin1(),zName.latin1());
  textEdit1->append(iQline);
  textEdit1->append(" -ndim        3  -aq2D          States                    \\");
  textEdit1->append(" -out ./data/test%03d.fid -verb -ov\n");

    
    
}


void frmmdd::ConvertMDD()
{
  QString sPipe3DName=lineEdit6->text().ascii();
  QString sMDDName=lineEdit7->text().ascii();
  QString sScheduleName=lineEdit8->text().ascii();
  QString Width=lineEdit18->text().ascii();
  QString Overlap=lineEdit19->text().ascii();
  int sys;
  QString MDDversion="0";
  QString sCommand;
  
  if (radioButton5->isChecked()) MDDversion="1";

  
  if (radioButton4->isChecked()) 
      sCommand="bruk2mdd "+sPipe3DName+" "+sMDDName+" "+MDDversion+" "+sScheduleName;
  if (radioButton5->isChecked())
      sCommand="pipe2multimdd "+sPipe3DName+" "+sMDDName+" "+Width+" "+Overlap+" "+sScheduleName;  
  sys=system(sCommand);
  qDebug("sys=%d", sys);
  qDebug(sCommand);
  if(sys==0)
  {
    QMessageBox::information( this, "MDD File Generator",
                              "Conversion appears to have finished nomally\n"
                              "Please check console window file for messages" );

  }
  else
  {
    QMessageBox::information( this, "MDD File Generator",
                              "Program does NOT seem to have completed normally\n"
                              "Check console window for messages" );
  }
  
  
  textEdit2->setText("In this step the user must start the actual MDD calculation.");
  textEdit2->append("For your convenience, you may enter the relevant parameters in the corresponding slots and then create a shell file with the appropriate command. However, you will still have to start the calculation manually.");
  
  if (radioButton4->isChecked()) {  // Parallel Calculation
    textEdit2->append("\nIf your dataset is large you may wish to login to the cluster. Please, check with your system administrators about the details of initiating a calculation on multiple processors\n");
    textEdit2->append("\nFor multiple processors, the command is 'mpirun -np <No of Procesors> mddnmrP_UHN <MDD File> <No. of Components> <Max Iterations> <Lambda> <random seed or initial solution> <reserved memory>'");
    textEdit2->append("	<No of Processors> - obvious, no? Anything above 1");
textEdit2->append("	<MDD File> is "+sMDDName);  
  }
  
  
  if (radioButton5->isChecked()) {  // Multiregion on Single Processor
    textEdit2->append("For multiregion processing, each region is calculated on a single processor, and the command is 'mddnmr1.2cx <MDD File> <No. of Components> 0 <Max Iterations> <Eps> <Lambda> <random seed or initial solution> <reserved memory>'");
    textEdit2->append("	<MDD File> is the *.mdd file in each of "+sMDDName+"_P1_P2 directories");
  }
  
  
    
  textEdit2->append("	<No. of Components> is the number of either 15N or 13C-HSQC peaks + 10-20% in most cases (such as 15N-NOESY-HSQC, CbCa(CO)NH or HCCH-TOCSY. It is a safe bet to have more components than necessary");
  
  if (radioButton5->isChecked()) textEdit2->append("	0 - a zero. No questions asked");
  
  textEdit2->append("	<Max Iterations> is the number of iterations you wish to perform. Somewhere between 250 and 500 is a good number. You can always continue with more iterations if necessary");
  if (radioButton5->isChecked()) textEdit2->append("	<Eps> is a convergence criterion. Use 1e-8");
  textEdit2->append("	<Lambda> is the Tikhonov regularization parameter. Use 0.01");
  textEdit2->append("	<random seed or initial solution> is either a file containing initial approximation (i.e. if you restart a calculation after a blackout or just want to add more iterations) or a random positive integer");
  textEdit2->append("	<reserved memory> is given in Mb. Necessary to store both the data and solution. Increase if your calculation crashes without starting the iterations.");

  textEdit2->append("\nThe result of the calculation is by default stored in file res.dat. Copy it to a file of your liking");
  textEdit2->append("This unfortunately puts a limitation - you may start only one calculation from each directory, as otherwise res.dat would get overwritten by multiple runs");
  textEdit2->append("res.dat may be used to restart the calculation as <random seed or initial solution> parameter above. However, due to slightly varying formats you cannot restart a multiprocessor run from a single processor result or vice versa");
  lineEdit8_2->setText(sPipe3DName);
  if (radioButton5->isChecked()) {
      sMDDName=lineEdit7->text()+".list";
      lineEdit9->setText(sMDDName.latin1());	      
  }
  else lineEdit9->setText("res.dat");	      
}


void frmmdd::Reconstruct()
{

  QString sPipe3DName=lineEdit8_2->text().ascii();
  QString sResName=lineEdit9->text().ascii();
  QString sFinalName=lineEdit10->text().ascii();
  QString sFinal3DName=sFinalName.section('.',0,0)+".ft3";
  QString sCommand;
  int sys;
  if (radioButton4->isChecked())
    sCommand="res2pipe "+sPipe3DName+" "+sResName+" "+sFinalName;
  if (radioButton5->isChecked())
    sCommand="multires2pipe "+sPipe3DName+" "+sResName+" "+sFinalName;
  sys=system(sCommand);
  if(sys==0)
  {
    QMessageBox::information( this, "Reconstructor",
                              "Program seems to have completed normally\n"
                              "Check console window for messages" );

  }
  else
  {
    QMessageBox::information( this, "Reconstructor",
                              "Program does NOT seem to have completed normally\n"
                              "Check console window for messages" );
  }

  textEdit3->setText("#!/bin/csh\n\n");

  textEdit3->append("# In this step the user has to use nmrPipe to Fourier Transform the indirect dimensions");
  textEdit3->append("# Use this template to change the required file names\n# and any other parameters to your liking.");
  textEdit3->append("# This script will be saved under standard file name 'proc_indirect.com'");
  textEdit3->append("# At this point you may also create your own script, as this is the final step.\n\n");

  
  
  textEdit3->append("nmrPipe -in "+sFinalName+" -verb                    \\");
  textEdit3->append("# Dimension X                                       \\");
  textEdit3->append("#| nmrPipe  -fn POLY -auto                          \\");
  textEdit3->append("| nmrPipe  -fn TP                                   \\");
  textEdit3->append("# Dimension Y                                       \\");
  textEdit3->append("#| nmrPipe  -fn LP -fb -auto                        \\");
  textEdit3->append("| nmrPipe  -fn SP -off 0.5 -end 0.98 -pow 2 -c 0.5  \\");
  textEdit3->append("| nmrPipe  -fn ZF -auto                             \\");
  textEdit3->append("| nmrPipe  -fn FT -auto                             \\");
  textEdit3->append("| nmrPipe  -fn PS -p0 0 -p1 0 -di -sw               \\");
  textEdit3->append("#| nmrPipe  -fn POLY -auto                          \\");
  textEdit3->append("| nmrPipe  -fn ZTP                                  \\");
  textEdit3->append("# Dimension Z                                       \\");
  textEdit3->append("#| nmrPipe  -fn LP -fb -auto                        \\");
  textEdit3->append("| nmrPipe  -fn SP -off 0.5 -end 0.99 -pow 2 -c 0.5  \\");
  textEdit3->append("| nmrPipe  -fn ZF -auto                             \\");
  textEdit3->append("| nmrPipe  -fn FT -auto                             \\");
  textEdit3->append("| nmrPipe  -fn PS -p0 0 -p1 0 -di -sw               \\");
  textEdit3->append("#| nmrPipe  -fn POLY -auto                          \\");
  if (radioButton4_2->isChecked())  
      textEdit3->append("| nmrPipe  -fn ZTP                                  \\");
  textEdit3->append("> "+sFinal3DName+" -verb\n\n");



}




void frmmdd::Bruk2Pipe()
{
    
  system("./fid_sp.com");
}


void frmmdd::SaveBruk2Pipe()
{
  FILE *fp;
  QString scripttext;
  if (NULL==(fp=fopen("fid_sp.com","w")))
  {
    QMessageBox::information( this, "Could not save file",
                              "fid_sp.com\n"
                              "Check your permissions" );
  }
  else
  {
    textEdit1->setTextFormat(PlainText);
    scripttext=textEdit1->text();
    fprintf(fp,"%s",scripttext.latin1());
    fclose(fp);
    system("chmod +x fid_sp.com");
    int b=scripttext.find("-out ",0,TRUE);
    int e=scripttext.find("\%03d.fid",0,TRUE);
    int l=scripttext.length();
    scripttext.remove(e,l);
    scripttext.remove(0,b+4);
	    
    textEdit4->setText("#!/bin/csh\n\n");
    textEdit4->append("# Use this script to process the first plane of the spectrum");
    textEdit4->append("# and determine the phase corrections in the directly detected dimension\n");
    textEdit4->append("# Adjust the file names, phases, apodization parameters and region limits");
    textEdit4->append("# Save and execute the script and use nmrDraw to view the output\n");
    textEdit4->append("# The script will be saved under standard name 'plane1.com'\n\n");
    textEdit4->append("\nnmrPipe -in "+scripttext+"001.fid -verb                   \\");
    textEdit4->append("# DO NOT DELETE OR MODIFY THIS COMMENT: BEGIN 1D     \\");
    textEdit4->append("#| nmrPipe  -fn SOL                                  \\");
    textEdit4->append("| nmrPipe  -fn SP -off 0.35 -end 0.98 -pow 2 -c 1.0  \\");
    textEdit4->append("| nmrPipe  -fn ZF -auto                              \\");
    textEdit4->append("| nmrPipe  -fn FT                                    \\");
    textEdit4->append("| nmrPipe  -fn PS -p0 0 -p1 0 -di -sw                \\");
    textEdit4->append("#| nmrPipe  -fn EXT -x1 10.5ppm -xn 6.0ppm  -sw      \\");
    textEdit4->append("# DO NOT DELETE OR MODIFY THIS COMMENT: END 1D       \\");
    textEdit4->append("| nmrPipe  -fn TP                                    \\");
    textEdit4->append("| nmrPipe  -fn SP -off 0.5 -end 0.98 -pow 2 -c 0.5   \\");
    textEdit4->append("| nmrPipe  -fn ZF -auto                              \\");
    textEdit4->append("| nmrPipe  -fn FT -auto                              \\");
    textEdit4->append("| nmrPipe  -fn PS -p0 0 -p1 0 -di -sw                \\");
    textEdit4->append("> ft/test001.ft2 -verb\n");


  }
}

void frmmdd::SavePlane1()
{
  FILE *fp;
  QString scripttext, fidname;
  if (NULL==(fp=fopen("plane1.com","w")))
  {
    QMessageBox::information( this, "Could not save file",
                              "plane1.com\n"
                              "Check your permissions" );
  }
  else
  {
    textEdit1->setTextFormat(PlainText);
    fidname=textEdit1->text();
    int b=fidname.find("-out ",0,TRUE);
    int e=fidname.find("-verb -ov",0,TRUE);
    int l=fidname.length();
    fidname.remove(e-1,l);
    fidname.remove(0,b+4);
    
    
    textEdit4->setTextFormat(PlainText);
    scripttext=textEdit4->text();
    fprintf(fp,"%s",scripttext.latin1());
    fclose(fp);
    system("chmod +x plane1.com");

    b=scripttext.find("# DO NOT DELETE OR MODIFY THIS COMMENT: BEGIN 1D",0,TRUE);
    e=scripttext.find("# DO NOT DELETE OR MODIFY THIS COMMENT: END 1D",0,TRUE);
    l=scripttext.length();
    scripttext.remove(e-1,l);
    scripttext.remove(0,b);
    textEdit6->setText("#!/bin/csh\n\n");
    textEdit6->append("# Use this script to process the directly detected dimension of the spectrum");
    textEdit6->append("# and to extract the region of interest for further processing\n");
    textEdit6->append("# Make sure to cut out water in 13C-aliphatic spectra\n");
    textEdit6->append("# Use the phase corrections from the previous step\n");
    textEdit6->append("# Adjust the file names, phases, apodization parameters and region limits");
    textEdit6->append("# Save and execute the script\n");
    textEdit6->append("# The script will be saved under standard name 'proc1d.com'\n\n");
    textEdit6->setBold ( TRUE );
    textEdit6->append("# Make sure not to change the output mode. I.e. DO NOT use pipe2xyz!\n\n");
    textEdit6->setBold ( FALSE );
    textEdit6->append("xyz2pipe -in "+fidname+" -verb            \\");
    textEdit6->append(scripttext);
    textEdit6->append("# DO NOT DELETE OR MODIFY THIS COMMENT: END 1D        \\");
    textEdit6->append("> ft/test_nus1.dat -verb\n");
  }
}


void frmmdd::ProcPlane1()
{
  
  system("mkdir -p ft");
  system("./plane1.com");

}


void frmmdd::NMRDraw()
{
  system("nmrDraw&");
}

void frmmdd::SaveProc1D()
{
  FILE *fp;
  QString scripttext,s;
  if (NULL==(fp=fopen("proc1d.com","w")))
  {
    QMessageBox::information( this, "Could not save file",
                              "proc1d.com\n"
                              "Check your permissions" );
  }
  else
  {
    textEdit6->setTextFormat(PlainText);
    scripttext=textEdit6->text();
    fprintf(fp,"%s",scripttext.latin1());
    fclose(fp);
    system("chmod +x proc1d.com");
    int b=scripttext.find("# DO NOT DELETE OR MODIFY THIS COMMENT: END 1D",0,TRUE);
    scripttext.remove(0,b);
    b=scripttext.find(">",0,TRUE);
    scripttext.remove(0,b);
    s=scripttext.section(' ',1,1);
    lineEdit6->setText(s);
  }

}


void frmmdd::Proc1D()
{
  system("./proc1d.com");

}



void frmmdd::SaveIndirect()
{
  FILE *fp;
  QString scripttext,s;
  if (NULL==(fp=fopen("proc_indirect.com","w")))
  {
    QMessageBox::information( this, "Could not save file",
                              "proc_indirect.com\n"
                              "Check your permissions" );
  }
  else
  {
    textEdit3->setTextFormat(PlainText);
    scripttext=textEdit3->text();
    fprintf(fp,"%s",scripttext.latin1());
    fclose(fp);
    system("chmod +x proc_indirect.com");
  }
}


void frmmdd::ProcIndirect()
{
  system("./proc_indirect.com");

}



void frmmdd::radioButton4_clicked()
{
// Parallel Computation
  radioButton4->setChecked(TRUE);
  radioButton5->setChecked(FALSE);
  lineEdit26->setText("32");
  lineEdit26->setReadOnly(FALSE);
  lineEdit22->setText("0 (ignored)");
  lineEdit22->setReadOnly(TRUE);
  textLabel10->setText("MDD Output File Name");
  lineEdit18->setText("0 (ignored)");
  lineEdit18->setReadOnly(TRUE);
  lineEdit19->setText("0 (ignored)");
  lineEdit19->setReadOnly(TRUE);
  textLabel2_2->setText("Output File from MDD Calculation");
  lineEdit7->setText("test.mdd");
}


void frmmdd::radioButton5_clicked()
{
// Single Processor
  radioButton5->setChecked(TRUE);
  radioButton4->setChecked(FALSE);
  lineEdit26->setText("1 (ignored)");
  lineEdit26->setReadOnly(TRUE);
  lineEdit22->setText("1e-8");
  lineEdit22->setReadOnly(FALSE);
  textLabel10->setText("MDD Output Directories\nCore Name");
  lineEdit18->setText("0.4");
  lineEdit18->setReadOnly(FALSE);
  lineEdit19->setText("0.1");
  lineEdit19->setReadOnly(FALSE);
  textLabel2_2->setText("File with List of Regions");
  lineEdit7->setText("mdd_out");
}


void frmmdd::CreateMDDScript()
{
  FILE *fp;
  QString sCommand;
  
  if (NULL==(fp=fopen(lineEdit27->text().ascii(),"w"))) {
    QMessageBox::information( this, "Could not save file",
                              lineEdit27->text().ascii(),
                              "\nCheck your permissions" );
    return;
  }
  if (radioButton5->isChecked()) {  // Single Processor
    fprintf(fp,"#!/bin/csh\n\n");
    fprintf(fp,"# This script gives the command line entry for running MDD calculations\n");
    fprintf(fp,"# on a single processor. Created by MDDGUI software\n");
    fprintf(fp,"# The general calculation command is \n");
    fprintf(fp,"# mddnmr1.2cx <data file> <Rank> 0 <MaxIter> <Eps> <Lambda> <ini file or seed> <memory Mb>\n");
    fprintf(fp,"# The script REQUIRES one argument - the list of regions produced when creating the MDD files\n");
    fprintf(fp,"# The file name of the list should be %s.list\n",lineEdit7->text().ascii());
    
    fprintf(fp,"# Check the readme files for MDDGUI or MDDNMR package for more information\n\n");
    

    fprintf(fp,"if( $# != 1 ) then\n");
    fprintf(fp,"  echo \"Usage:\" $0 \"<File name>\"\n");
    fprintf(fp,"  echo \"  The file should contain the list of regions for calculation\"\n");
    fprintf(fp,"  cat $0\n");  
    fprintf(fp,"  exit  \n");
    fprintf(fp,"endif\n");
    fprintf(fp,"set regions=`grep -v '#' $1 | gawk '{print $1}'`\n");
    sCommand="mddnmr1.2c $mdd "+lineEdit20->text()+" 0 "+lineEdit21->text()+" "+lineEdit22->text()+" "+lineEdit23->text()+" "+lineEdit24->text()+" "+lineEdit25->text();


    fprintf(fp,"foreach region ( $regions )\n");
    fprintf(fp,"  echo now starting $region\n");
    fprintf(fp,"  cd $region\n");
    fprintf(fp,"  set mdd=$region'.mdd'\n");
    fprintf(fp,"  echo %s > out.log \n",sCommand.latin1());
    fprintf(fp,"  sleep 6\n");
    fprintf(fp,"  date >> out.log\n");
    fprintf(fp,"  %s >> out.log &\n",sCommand.latin1());
    fprintf(fp,"  cd ..\n");
    fprintf(fp,"end\n\n");
    
	
  }
  if (radioButton4->isChecked()) {  // Parallel Computation
    fprintf(fp,"#!/bin/sh\n\n");
    fprintf(fp,"# This script gives the command line entry for running MDD calculations\n");
    fprintf(fp,"# on a multiple processors. Created by MDDGUI software\n");
    fprintf(fp,"# The general command is \n");
    fprintf(fp,"# mpirun -np <Processors> mddnmrP_UHN <data file> <Rank> <MaxIter> <Lambda> <ini file or seed> <memory Mb>\n");
    fprintf(fp,"# Check the readme files for MDDGUI or MDDNMR package for more information\n\n");
    fprintf(fp,"# ***You first have to log on to the cluster and start the parallel ring***\n\n");
    fprintf(fp,"# At UHN:\n# 1. rlogin quasar (or pulsar)\n# 2. mpdboot2\n\n");
    fprintf(fp,"mpirun -np %s mddnmrP_UHN %s %s %s %s %s %s\n", lineEdit26->text().ascii(), lineEdit7->text().ascii(), lineEdit20->text().ascii(), lineEdit21->text().ascii(), lineEdit23->text().ascii(), lineEdit24->text().ascii(), lineEdit25->text().ascii());
	
  }
  fclose(fp);
  sCommand="chmod +x "+lineEdit27->text();
  system(sCommand);
}


void frmmdd::BrukerButtonClicked()  //Button 3
{
  radioButton3->setChecked(TRUE);
  radioButton4_2->setChecked(FALSE);
  textLabel2->setText("Recorded 'ser' File");
  leInputSer->setText("ser");
  leInputSer->setReadOnly(FALSE);
  textLabel3->setText("Variable Delay List");
  leVDlist->setText("vdlist");
  leVDlist->setReadOnly(FALSE);
  leOutSer->setText("ser.sp");
  leOutSer->setReadOnly(FALSE);
   

}


void frmmdd::VarianButtonClicked()  //Button 4_2
{
  radioButton3->setChecked(FALSE);
  radioButton4_2->setChecked(TRUE);
  textLabel2->setText("Recorded 'fid' File");
  leInputSer->setText("fid");
  leInputSer->setReadOnly(FALSE);
  textLabel3->setText("Varian procpar file");
  leVDlist->setText("procpar");
  leVDlist->setReadOnly(FALSE);
  leOutSer->setText("fid.sp");
  leOutSer->setReadOnly(FALSE);

}
