#include <stdlib.h>
#include <stream.h>
#include <math.h>

#define MAXSTR 1024

int main(int argc, char **argv)
{
  float header[512];
  int m,n,k;
  int i,j,l,q;
  int m0,n0,k0,r0,d0;
  int newformat=0;
  int p1,p2,q1,q2;
  int overlap=4;
  float *data=NULL;
  float *s1;
  float *s2;
  float *s3;
  float *a;
  float z;
  FILE *fr;
  FILE *fp;
  FILE *fl;
  char fpname[MAXSTR];
  char foname[MAXSTR];
  char frname[MAXSTR];
  char flname[MAXSTR];
  char dirname[MAXSTR];
  char fake;
  char iline[MAXSTR];
  char *p;
  
  if (argc!=4) {
    cout<<"Usage: res2pipe <nmrPipe stream> <list of regions> <nmrPipe output>\n";
    cout<<"            <nmrPipe stream> - file (not planes!) used to create MDD input\n";
    cout<<"            <list of regions>- file containing the list of directories and point boundaries\n";
    cout<<"              it is assumed that the result of each calculation is stored in res.dat in the correct subdirectory\n";
    cout<<"            <nmrPipe output> - output file with reconstructed (but not FT'ed) spectrum\n";
    exit(1);
  }
  strcpy(fpname,argv[1]);
  strcpy(flname,argv[2]);
  strcpy(foname,argv[3]);
  
  if (NULL==(fp=fopen(fpname,"rb"))) {
    cerr<<"Could not open file "<<fpname<<" for reading. Exiting...\n";
    exit(1);
  }
  if (512!=fread(header,sizeof(float),512,fp)) {
    cerr<<"Could not read 512 floats into header from file "<<fpname<<". Exiting...\n";
    exit(1);
  }  
  k=header[15];    // dim 3 size
  n=header[428]*2; // dim 2 size
  m=header[99];    // dim 1 size
  
  cout<<"\nData sizes are X: "<<m<<" Y: "<<n<<" Z: "<<k<<"\n";
  if (NULL==(data=new float[m*n*k])) {
    cerr<<"Could not allocate "<<m*n*k*4<<" bytes of memory to store data. Exiting...\n";
    exit(1);
  }
  
  
  if (m*n*k!=fread(data,sizeof(float),m*n*k,fp)) {    
    cerr<<"Could not read "<<m*n*k<<" floats of data from file "<<fpname<<". Exiting...\n";
    exit(1);
  }
  if (0<fread(&fake,1,1,fp)) {
    cerr<<"There is still data in file "<<fpname<<" although "<<512+m*n*k<<" floats of header and data were read. Exiting..\n";
    exit(1);
  }
  fclose(fp);
  
  
// opening the list of regions
  if (NULL==(fl=fopen(flname,"r"))) {
    cout<<"Could not open file "<<frname<<". Exiting...\n";
    exit(1);
  }

  
  for (p=fgets(iline,MAXSTR,fl);p!=NULL;p=fgets(iline,MAXSTR,fl)) {
    if ('#'==iline[0]) {
      if (NULL!=strstr(iline,"Overlap points")) {
        if (1!=sscanf(iline+17,"%d",&overlap)) {
          cerr<<"Could not determine the number of overlap points: \n   "<<iline<<"\nExiting...";
          exit(1);
        }
      }
      continue;
    }
    if (3!=sscanf(iline,"%s %d %d",dirname,&p1,&p2)) {
      cerr<<"wrong file format in line "<<iline<<"\n";
      exit(1);
    }
    sprintf(frname,"%s/res.dat",dirname);
  
  
// opening individual region
    if (NULL==(fr=fopen(frname,"r"))) {
      cout<<"Could not open file "<<frname<<". Exiting...\n";
      exit(1);
    }

  
  //***************** Checking for sizes from res.dat ****************

    do {
      fscanf(fr,"%c",&fake);
    }
    while ('$'!=fake);
    if (5!=fscanf(fr,"%d %*d %d %d %d %d",&d0,&r0,&m0,&n0,&k0)) {
      cout<<"Could not read number of dimensions from result file "<<frname<<". Exiting...\n";
      exit(4);
    }
    else {
      if (d0!=3) {
        cout<<"Result file "<<frname<<" indicates "<<d0<<" dimensions. Currently only 3D spectra are supported. Exiting...\n";
        exit(4);
      }
    }
    
    // ***************** Memory allocations ******************
    if (NULL==(s1=new float[m0*r0])) {
      cerr<<"Could not allocate "<<m0*r0*4<<" bytes of memory to store shape s1. Exiting...\n";
      exit(2);
    }
    if (NULL==(s2=new float[n0*r0])) {
      cerr<<"Could not allocate "<<n0*r0*4<<" bytes of memory to store shape s2. Exiting...\n";
      exit(2);
    }
    if (NULL==(s3=new float[k0*r0])) {
      cerr<<"Could not allocate "<<k0*r0*4<<" bytes of memory to store shape s3. Exiting...\n";
      exit(2);
    }
    if (NULL==(a=new float[r0])) {
      cerr<<"Could not allocate "<<r0*4<<" bytes of memory to store amplitudes. Exiting...\n";
      exit(2);
    }

    // ******************** Reading the actual data *********************
    // amplitudes come first
    for (j=0; j<r0; j++)
      fscanf(fr,"%f",a+j);
    for (i=0; i<m0; i++)
      for (j=0; j<r0; j++)
        if (1!=fscanf(fr,"%f",s1+i*r0+j)) {
          cout<<"Could not read "<<i<<" element of "<<j<<" component (shape 1) from file "<<frname<<"\n";
	  exit(3);
        }
    for (i=0; i<n0; i++)
      for (j=0; j<r0; j++)
        if (1!=fscanf(fr,"%f",s2+i*r0+j)) {
          cout<<"Could not read "<<i<<" element of "<<j<<" component (shape 2) from file "<<frname<<"\n";
	  exit(3);
        }
    for (i=0; i<k0; i++)
      for (j=0; j<r0; j++)
        if (1!=fscanf(fr,"%f",s3+i*r0+j)) {
          cout<<"Could not read "<<i<<" element of "<<j<<" component (shape 3) from file "<<frname<<"\n";
	  exit(3);
        }
    fclose(fr);
 
    cout<<"Region "<<dirname<<" sizes are X: "<<m0<<"; Y: "<<n0<<"; Z: "<<k0<<"; Number of components: "<<r0<<"\n";
 
  
  
  
    if (p1>1) q1=p1+overlap;
    else q1=p1;
    if (p2<m) q2=p2-overlap;
    else q2=p2;
    for (i=0; i<k; i++) 
      for (j=0; j<n; j++) 
        for (l=q1-1; l<q2; l++) 
          if (fabs(data[i*m*n+j*m+l])<1e-6) {
	    data[i*m*n+j*m+l]=0;
	    for (q=0; q<r0; q++) 
	      data[i*m*n+j*m+l]+=a[q]*s1[(l-p1+1)*r0+q]*s2[j*r0+q]*s3[i*r0+q];
	  }
    delete[] a;
    delete[] s1;
    delete[] s2;
    delete[] s3;
    a=NULL;
    s1=NULL;
    s2=NULL;
    s3=NULL;
  }
  if (NULL==(fp=fopen(foname,"wb"))) {
    cerr<<"Could not open file "<<foname<<" for writing. Exiting...\n";
    exit(1);
  }
  if (512!=fwrite(header,sizeof(float),512,fp)) {
    cerr<<"Could not write 512 floats of header into file "<<foname<<". Exiting...\n";
    exit(1);
  }
  if (m*n*k!=fwrite(data,sizeof(float),m*n*k,fp)) {    
    cerr<<"Could not write "<<m*n*k<<" floats of data into file "<<foname<<". Exiting...\n";
    exit(1);
  }
  
  fclose(fp);
  delete[] data;
  
  cerr<<"done...\n";
  return 0;
}
