#include <stdlib.h>
#include <stream.h>
#include <math.h>

#define MAXSTR 1024

int main(int argc, char **argv)
{
  float header[512];
  int m,n,k;
  int l,j,i;
  int nregions,m1,mN,overlap;
  int p1,p2,mm,n2;
  int ncount,cnt;
  int data_count,index_count;
  int ns,ni,nj,nr;
  int checkflg=0;
  float *data=NULL;
  int *sel=NULL;
  FILE *fm;
  FILE *fp;
  FILE *fs;
  FILE *fr;
  char fpname[MAXSTR];
  char fmname[MAXSTR];
  char fsname[MAXSTR];
  char corename[MAXSTR];
  char dirname[MAXSTR];
  char scommand[MAXSTR];
  char frname[MAXSTR];
  char fake;
  double width,xSW;
  double overlap_width;
  

  if (argc<5 || argc > 6) {
    cout<<"Usage: bruk2multimdd <nmrPipe stream> <core output> <width of regions> <overlap> [<schedule>]\n";
    cout<<"                <nmrPipe stream> - file (not planes!) containing 3D nmrPipe format data\n";
    cout<<"                <core output>    - output directory name core\n";
    cout<<"                <region width>   - total number of regions in ppm, N\n";
    cout<<"                <overlap>        - width in ppm on each edge of each region that is discarded\n";
    cout<<"                <schedule>       - schedule (selection) file, optional but may be useful for a double check\n";
    exit(1);
  }
  
  strcpy(fpname,argv[1]);
  strcpy(corename,argv[2]);
  if (1!=sscanf(argv[3],"%lf",&width)) {
          cerr<<"Could not determine region width ["<<argv[3]<<"]. Exiting...\n";
    exit(1);
  }
  if (1!=sscanf(argv[4],"%lf",&overlap_width)) {
          cerr<<"Could not determine width of overlap ["<<argv[4]<<"]. Exiting...\n";
    exit(1);
  }


  if (argc==6) {
    checkflg=1;
    strcpy(fsname,argv[5]);
  }
  
  
  
  if (NULL==(fp=fopen(fpname,"rb"))) {
    cerr<<"Could not open file "<<fpname<<" for reading. Exiting...\n";
    exit(2);
  }
  
  if (512!=fread(header,sizeof(float),512,fp)) {
    cerr<<"Could not read 512 floats into header from file "<<fpname<<". Exiting...\n";
    exit(3);
  }  
  
  k=header[15];    // dim Z size
  n=header[428]*2; // dim Y size
  m=header[99];    // dim X size
  
  xSW=header[100]/header[119]; // dim X in PPM
  cout<<"xSW= "<<xSW<<"\n";
  nregions=((xSW-overlap_width)/(width-overlap_width)+0.5);
  overlap=m*0.5*overlap_width/xSW+0.5;
  
  overlap_width=xSW*2.0*overlap/m;
  width=(xSW+(nregions-1)*overlap_width)/nregions;
  
  m1=(m+(nregions-1)*overlap*2)/nregions;
  mN=m-(nregions-1)*(m1-overlap*2);
  n2=mN-m1;
  
  cout<<"\nSizes are X: "<<m<<"; Y: "<<n<<"; Z: "<<k<<"\n";
  cout<<"Total number of regions: "<<nregions<<"\n";
  cout<<"Region width and overlap width have been adjusted to have integer number of points.\nwidth: "<<width<<" and overlap: "<<overlap_width<<"\n";
  if (n2>0) 
    cout<<"Regions 1 to "<<nregions-n2<<": "<<m1<<" points and regions "<<nregions-n2+1<<" to "<<nregions<<": "<<m1+1<<" points\n";
  else 
    cout<<"Regions 1 to "<<nregions<<": "<<m1<<" points\n";
  cout<<"Overlap between regions: "<<overlap<<"*2="<<overlap*2<<" points\n";
  
  if (NULL==(data=new float[m*n*k])) {
    cerr<<"Could not allocate "<<m*n*k*4<<" bytes of memory to store data. Exiting...\n";
    exit(4);
  }
    
  if (m*n*k!=fread(data,sizeof(float),m*n*k,fp)) {    
    cerr<<"Could not read "<<m*n*k<<" floats of data from file "<<fpname<<". Exiting...\n";
    exit(3);
  }
  if (0<fread(&fake,1,1,fp)) {
    cerr<<"There is still data in file "<<fpname<<" although "<<512+m*n*k<<" floats of header and data were read. Exiting..\n";
    exit(3);
  }
  fclose(fp);
  
  cout<<"checkflg="<<checkflg<<"\n";
  
  if (NULL==(sel=new int[n*k])) {
    cerr<<"Could not allocate "<<n*k*sizeof(int)<<" bytes of memory to store selection matrix. Exiting...\n";
    exit(4);
  }

  for(i=0; i<k; i++)
    for (j=0; j<n; j++)
      sel[i*n+j]=0;
  
  if (checkflg) {
    if (NULL==(fs=fopen(fsname,"r"))) {
      cerr<<"Could not open file "<<fsname<<" for reading. Exiting...\n";
      exit(2);
    }
 
    for(ns=fscanf(fs,"%d%d",&ni,&nj); ns==2; ns=fscanf(fs,"%d%d",&ni,&nj)) {
      sel[ 2*ni   *n +2*nj  ]=1;
      sel[(2*ni+1)*n +2*nj  ]=1;
      sel[ 2*ni   *n +2*nj+1]=1;
      sel[(2*ni+1)*n +2*nj+1]=1;
    }
    fclose(fs);
    ncount=0;
    cnt=0;
  
    for (i=0; i<k; i++) 
      for (j=0; j<n; j++) {
        if (fabs(data[i*m*n+j*m])>1e-6) {
	  ncount++;
	  if (sel[i*n+j]==0)
	    cerr<<i<<" "<<j<<" recorded unexpectedly\n";
        }
        else
          if (sel[i*n+j]==1)
 	    cerr<<i<<" "<<j<<" expected but not recorded\n";
        if (sel[i*n+j])
          cnt++;
        }
  }
  else {
    ncount=0;
    cnt=0;
    for (i=0; i<k; i++) 
      for (j=0; j<n; j++) 
        if (fabs(data[i*m*n+j*m])>1e-6) {
	  sel[i*n+j]=1;
	  ncount++; cnt++;
	}
  
  }
  cout<<"\ncounted "<<cnt<<"; "<<ncount<<" fids\n";
//  writing out region files
  sprintf(frname,"%s.list",corename);
  if (NULL==(fr=fopen(frname,"w"))) {
    cerr<<"Could not open file "<<fmname<<" for writing. Exiting...\n";
    exit(2);
  }
  fprintf(fr,"# List of regions with first and last points\n");
  fprintf(fr,"# Total regions   %d\n",nregions);
  fprintf(fr,"# Overlap points   %d\n",overlap);
  fprintf(fr,"# <Region directory> <first point> <last point>\n");
    
  for (nr=0;nr<nregions;nr++) {
    if (nr<nregions-n2) {
      p1=nr*(m1-overlap*2);
      p2=p1+m1;
      mm=m1;
    }
    else {
      p1=(nregions-n2)*(m1-overlap*2)+(nr+n2-nregions)*(m1+1-overlap*2); 
      p2=p1+m1+1;
      mm=m1+1;
    }
    sprintf(dirname,"%s_%03d_%03d",corename,p1+1,p2);
    sprintf(scommand,"mkdir -p %s",dirname);
    system(scommand);
    cout<<"creating "<<dirname<<"\n";
    sprintf(fmname,"%s/%s.mdd",dirname,dirname);
    if (NULL==(fm=fopen(fmname,"w"))) {
      cerr<<"Could not open file "<<fmname<<" for writing. Exiting...\n";
      exit(2);
    }
    fprintf(fr,"%s %15d %13d\n",dirname,p1+1,p2);
    fprintf(fm,"mdd sparse asc\n");
    fprintf(fm,"produced by %s from file %s\n",argv[0],fpname);
    fprintf(fm,"region %03d: points %d to %d\n",nr+1,p1+1,p2);
    if (checkflg) fprintf(fm,"checked against file %s for consistency\n",fsname);
    fprintf(fm,"$\n3 1\n");
    fprintf(fm,"%d %d %d %d\n",mm*cnt,mm,n,k);
    index_count=0;
    for (i=0; i<k; i++) 
      for (j=0; j<n; j++) 
        if (sel[i*n+j]) 
          for (l=0; l<mm; l++) {
	    fprintf(fm,"%d %d %d\n",l,j,i);
	    index_count++;
	  }
	    
    data_count=0;
    for (i=0; i<k; i++) 
      for (j=0; j<n; j++) 
        if (sel[i*n+j]) 
          for (l=p1; l<p2; l++) {
	    fprintf(fm,"%e\n",data[i*m*n+j*m+l]);
	    data_count++;
	  }
    
    cout<<"  data_count="<<data_count<<" index_count="<<index_count<<" fid_count="<<mm*cnt<<"\n";
    fclose(fm);
  } 
  fclose(fr);
  cerr<<"\ndone...\n";
  return 0;
}
