// sparsebruker.cpp
// AG 2005
// Inserts zeros into Bruker ser file 
// in order for it to be processed by nmrPipe

#include <stdlib.h>
#include <stream.h>
#include <stdio.h>
#include <math.h>

#define MAXSTR 1024


int main(int argc, char **argv)
{

  FILE *fs;
  FILE *fv;
  FILE *fp;

  char sname[MAXSTR];
  char vname[MAXSTR];
  char pname[MAXSTR];
  char vline[MAXSTR];

  int fidsz;
  int i,j,l,zr;

//  int *zero;
//  int *fid;
//  cout<<sizeof(int);

  if (argc<4 || argc>5) {
    cout<<"Usage: sparsebruker <ser> <vdlist> <ser.sp> [<fid size>] \n";
    cout<<"       <ser>      Bruker ser file\n";
    cout<<"       <vdlist>   variable delay list (3u = point recorded; 1u = point NOT recorded)\n";
    cout<<"       <ser.sp>   ser file with zeros in correct places\n";
    cout<<"       <fid size> number of points in each Bruker fid (check parameter TD in acqu, default 1024)\n";
    exit(1);
  }

  
  strcpy(sname,argv[1]);
  strcpy(vname,argv[2]);
  strcpy(pname,argv[3]);
  if (argc==5) {
    if (1!=sscanf(argv[4],"%d",&fidsz)) {
      cout<<"Wrong parameter 4: "<<argv[4]<<". Exiting...\n";
      exit(1);
    }
    if (fidsz<0) {
      cout<<"Wrong parameter 4: "<<argv[4]<<". Exiting...\n";
      exit(1);
    }
  }
  else fidsz=1024;
  
  cout<<fidsz<<"\n";
  
  int *zero = new int[fidsz*2];
  for (i=0; i<fidsz*2; i++) zero[i]=0;
  int *fid  = new int[fidsz*2];

  if (NULL==(fs=fopen(sname,"rb"))) {
    cout<<"Could not open file "<<sname<<" for reading. Exiting...\n";
    exit(2);
  }
  if (NULL==(fv=fopen(vname,"r"))) {
    cout<<"Could not open file "<<vname<<" for reading. Exiting...\n";
    exit(2);
  }
  if (NULL==(fp=fopen(pname,"wb"))) {
    cout<<"Could not open file "<<sname<<" for writing. Exiting...\n";
    exit(2);
  }

  i=0;
  j=0;
  while (NULL!=fgets(vline,MAXSTR,fv)) {
    j++;
    if (NULL==index(vline,'3')) {
      if (fidsz*2!=fwrite(zero,4,2*fidsz,fp)) {
        cout<<"Could not write 2*"<<fidsz<<" zeros into file "<<pname<<". Exiting...\n";
	exit(3);
      }
    }
    else {
      i++;
      if (fidsz*2!=fread(fid,4,2*fidsz,fs)) {
        cout<<"Could not read 2*"<<fidsz<<" points from file "<<sname<<". Exiting...\n";
	exit(3);
      }
      if (fabs(fid[2*fidsz-5])<1e-6) {
        zr=1;
        for (int l=0;l<fidsz*2;l++) 
	  if (fid[l]>1e-6) zr=0;
	if (zr) cout<<"Looks like entire fid is zero at "<<i<<", "<<j<<"\n";
      }	
      if (fidsz*2!=fwrite(fid,4,2*fidsz,fp)) {
        cout<<"Could not write 2*"<<fidsz<<" points into file "<<pname<<". Exiting...\n";
	exit(3);
      }
    }
  }
  cout<<i*2<<" fids read\n";
  if (fidsz*2!=fread(fid,4,2*fidsz,fs)) 
    cout<<"Are you sure that was a sparse spectrum?\n";
  else if (fabs(fid[2*fidsz-1])>1e-6) 
    cout<<"Looks like there is non-zero data left in file "<<sname<<"\n";
    
  fclose(fs);
  fclose(fv);
  fclose(fp);
  
//  delete fid;
//  delete zero;
  
  cout<<"\ndone...\n";
  
  return(0);
}
