// sparsebruker.cpp
// AG 2005
// Inserts zeros into Varian fid file 
// in order for it to be processed by nmrPipe

#include <stdlib.h>
#include <stream.h>
#include <stdio.h>
#include <math.h>
#include <netinet/in.h>
#include "data.h"

#define MAXSTR 1024

void getpar(char *key, FILE *fp, char *result);

int main(int argc, char **argv)
{

  FILE *ff;
  FILE *fp;
  FILE *fn;
  FILE *fs;

  char fname[MAXSTR];
  char pname[MAXSTR];
  char sname[MAXSTR];
  char nname[MAXSTR];
  char sline[MAXSTR];
  char rline[MAXSTR];
  char *s;

  int i,ix,id2,id3,id4,ph,ph2,ph3;
  int ni,ni2,nimax,nimax2,np, fidcount;

  dfilehead fhead;
  dblockhead bhead;
  
  if (argc!=4) {
    cout<<"Usage: sparsevarian <fid> <procpar> <fid.sp>  \n";
    cout<<"       <fid>         Varian fid file\n";
    cout<<"       <procpar>     Varian procpar file\n";
    cout<<"       <fid.sp>      fid file with zeros in correct places\n";
    exit(1);
  }

  
  strcpy(fname,argv[1]);
  strcpy(pname,argv[2]);
  strcpy(nname,argv[3]);
    
  if (NULL==(ff=fopen(fname,"rb"))) {
    cout<<"Could not open file "<<fname<<" for reading. Exiting...\n";
    exit(2);
  }
  if (NULL==(fp=fopen(pname,"r"))) {
    cout<<"Could not open file "<<pname<<" for reading. Exiting...\n";
    exit(2);
  }
  if (NULL==(fn=fopen(nname,"wb"))) {
    cout<<"Could not open file "<<nname<<" for writing. Exiting...\n";
    exit(2);
  }

  if (1 != fread( &fhead, sizeof(dfilehead), 1, ff)) {
    cout<<"Can't read file header from the FID file "<<fname<<". Exiting...\n";
    exit(2);
  }
  
  getpar("ni",fp,rline);
  sscanf(rline,"%d",&ni);
  getpar("ni2",fp,rline);
  sscanf(rline,"%d",&ni2);
  getpar("nimax",fp,rline);
  sscanf(rline,"%d",&nimax);
  getpar("nimax2",fp,rline);
  sscanf(rline,"%d",&nimax2);
  getpar("np",fp,rline);
  sscanf(rline,"%d",&np);
  
  getpar("sparse_file",fp,rline);
  for (i=0; i<strlen(rline); i++)
    if (rline[i]=='"') rline[i]=' ';
  sscanf(rline,"%s",sname);
  strcat(sname,".hdr_3");
  
  if (NULL==(fs=fopen(sname,"r"))) {
    cout<<"Could not open file "<<sname<<" for reading. Exiting...\n";
    exit(2);
  }
  
  if (ntohl(fhead.np)!=np) {
    cout<<"Warning: fid ("<<ntohl(fhead.np)<<") and procpar ("<<np<<") have different values for np. Continuing with fid value\n";
    np=ntohl(fhead.np);
  }
  if (ntohl(fhead.nblocks)!=ni*ni2*4) {
    cout<<"Warning: fid ("<<ntohl(fhead.nblocks)<<") and procpar ("<<ni*ni2*4<<") have different values for number of fids recorded. Exiting...\n";
    exit(1);
  }
  
  cout<<"\nLength of individual fid: "<<np<<" points\n";
  
  
  int *data = new int[np*nimax*nimax2*4];
  for (i=0; i<np*nimax*nimax2*4; i++) data[i]=0;
  
  fidcount=0;
  for (s=fgets(sline, MAXSTR, fs);(s!=NULL);s=fgets(sline, MAXSTR, fs)) {
    if (sline[0]=='#') continue;
    if (2!=sscanf(sline,"%d%d",&id2,&id3)) {
      cout<<"Cannot read 2 ingers from line "<<sline<<" of file "<<sname<<".Exiting...\n";
      exit(2);
    }
    for (ph2=0; ph2<2; ph2++) {
      for (ph=0; ph<2; ph++) { 
        if (1 != fread( &bhead, sizeof(dblockhead), 1, ff)) {
          cout<<"Can't read block header "<<fidcount<<" from the FID file "<<fname<<". Exiting...\n";
          exit(2);
        }
    
        if (np != fread( data+((id2+id3*nimax)*4+ph+ph2*2)*np, ntohl(fhead.ebytes), np, ff)) {
          cout<<"Can't read fid "<<fidcount<<" from the FID file "<<fname<<". Exiting...\n";
          exit(2);
        }
        fidcount++;
      }
    }
    
  }
  cout<<"Number of fids successfully read from fid file: "<<fidcount<<" out of "<<ni*ni2*4<<" expected\n";
  
  fhead.nblocks=htonl(nimax*ni2*4);
  if (1 != fwrite( &fhead, sizeof(dfilehead), 1, fn)) {
    cout<<"Can't write file header to the FID.SP file "<<nname<<". Exiting...\n";
    exit(2);
  }
  fidcount=0;
  for (i=0;i<nimax*ni2*4;i++) {
    bhead.index=htonl(i+1);
    if (1 != fwrite( &bhead, sizeof(dblockhead), 1, fn)) {
      cout<<"Can't write block header "<<i+1<<" to the FID.SP file "<<nname<<". Exiting...\n";
      exit(2);
    }
    if (np != fwrite( data+i*np, ntohl(fhead.ebytes), np, fn)) {
      cout<<"Can't write fid # "<<i+1<<" to the FID.SP file "<<nname<<". Exiting...\n";
      exit(2);
    }
    fidcount++;
  }
  cout<<"Number of fids successfully written into "<<nname<<" file: "<<fidcount<<" out of "<<nimax*ni2*4<<" expected\n";
  
  delete[] data;
  fclose(fs);
  fclose(fp);
  fclose(fn);
  fclose(ff);
  
/*  if (NULL==(fc=fopen(sname,"w"))) {
    cout<<"Could not open file "<<cname<<" for writing. Exiting...\n";
    exit(2);
  }
  fprintf(fc,"#!/bin/csh\n\n");
  fprintf(fc,"# var2pipe conversion script for sparse 3D spectra\n");
  fprintf(fc,"#    1. Check the input file (%s)\n",nname);
  fprintf(fc,"#    2. Check the acquisition modes for Y and Z (can't guess Echo-Antiecho)\n");
  fprintf(fc,"#    3. Modify observation frequencies, carrier positions and axes names as necessary\n");
  fprintf(fc,"#    4. Number of points and sweep width values should be correct, but check anyway\n\n");*/
  
  cout<<"\ndone...\n";
  
  return(0);
  
}

void getpar(char *key, FILE *fp, char *result)
{
  char res[MAXSTR];
  char word[MAXSTR];
  char iline[MAXSTR];
  char *p;
  int found=0,n;


  rewind(fp);
  strcpy(res,"");
  for(p=fgets(iline, MAXSTR, fp);(p!=NULL && found==0);p=fgets(iline, MAXSTR, fp)) {
    sscanf(iline,"%s",word);
    if (0==strcmp(word,key)) {
      p=fgets(iline,MAXSTR,fp);
      found=1;
      sscanf(iline,"%s",word);
      n=strlen(word);
      strcpy(res,iline+n+1);
    }
  }
  strcpy(result,res);
}
